@@ -23,15 +23,16 @@
 #ifndef __PANGOFT2_H__
 #define __PANGOFT2_H__
 
-#include <freetype/freetype.h>
-
 #include <fontconfig/fontconfig.h>
 
 #include <pango/pango-layout.h>
+#include <pango/pangofc-font.h>
 
 G_BEGIN_DECLS
 
+#ifndef PANGO_DISABLE_DEPRECATED
 #define PANGO_RENDER_TYPE_FT2 "PangoRenderFT2"
+#endif
 
 #define PANGO_TYPE_FT2_FONT_MAP              (pango_ft2_font_map_get_type ())
 #define PANGO_FT2_FONT_MAP(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FT2_FONT_MAP, PangoFT2FontMap))
@@ -43,10 +44,6 @@ typedef void (*PangoFT2SubstituteFunc) (FcPattern *pattern,
                                         gpointer   data);
 
 /* Calls for applications */
-PangoContext      *pango_ft2_get_context          (double dpi_x,
-						   double dpi_y);
-PangoFontMap      *pango_ft2_font_map_for_display (void);
-void               pango_ft2_shutdown_display     (void);
 
 
 void           pango_ft2_render             (FT_Bitmap        *bitmap,
@@ -79,6 +76,12 @@ PangoContext *pango_ft2_font_map_create_context         (PangoFT2FontMap
 
 /* API for rendering modules
  */
+#ifndef PANGO_DISABLE_DEPRECATED
+PangoContext      *pango_ft2_get_context          (double dpi_x,
+						   double dpi_y);
+PangoFontMap      *pango_ft2_font_map_for_display (void);
+void               pango_ft2_shutdown_display     (void);
+
 PangoGlyph     pango_ft2_get_unknown_glyph (PangoFont       *font);
 int            pango_ft2_font_get_kerning  (PangoFont       *font,
 					    PangoGlyph       left,
@@ -86,6 +89,7 @@ int            pango_ft2_font_get_kerning  (PangoFont       *font,
 FT_Face        pango_ft2_font_get_face     (PangoFont       *font);
 PangoCoverage *pango_ft2_font_get_coverage (PangoFont       *font,
 					    PangoLanguage   *language);
+#endif /* PANGO_DISABLE_DEPRECATED */
 
 G_END_DECLS
 
